"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createController = createController;
const common_1 = require("@nestjs/common");
const utils_1 = require("./utils");
const constants_1 = require("../constants");
const json_base_controller_1 = require("../mixin/controller/json-base.controller");
const interceptors_1 = require("../mixin/interceptors");
function createController(entity, controller) {
    const controllerClass = controller ||
        (0, utils_1.nameIt)((0, utils_1.getProviderName)(entity, constants_1.JSON_API_CONTROLLER_POSTFIX), json_base_controller_1.JsonBaseController);
    const entityName = entity instanceof Function ? entity.name : entity.options.name;
    if (!Object.prototype.isPrototypeOf.call(json_base_controller_1.JsonBaseController, controllerClass)) {
        throw new Error(`Controller "${controller?.name}" should be inherited of "JsonBaseController"`);
    }
    const decoratorOptions = Reflect.getMetadata(constants_1.JSON_API_DECORATOR_OPTIONS, controllerClass);
    (0, common_1.Controller)(decoratorOptions?.['overrideRoute'] || `${(0, utils_1.camelToKebab)(entityName)}`)(controllerClass);
    (0, common_1.Inject)(constants_1.TYPEORM_SERVICE)(controllerClass.prototype, constants_1.TYPEORM_SERVICE_PROPS);
    (0, common_1.UseInterceptors)(interceptors_1.LogTimeInterceptors)(controllerClass);
    (0, common_1.UseInterceptors)(interceptors_1.ErrorInterceptors)(controllerClass);
    return controllerClass;
}
//# sourceMappingURL=create-controller.js.map