"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresError = exports.MysqlError = void 0;
const utils_1 = require("./utils");
const common_1 = require("@nestjs/common");
const fieldNotNullOrDefault = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: (0, utils_1.formErrorString)(entityMetadata, errorText),
        path: ['data', 'attributes'],
    };
    return new common_1.BadRequestException([error]);
};
const duplicateItems = (entityMetadata, errorText, detail) => {
    errorText = 'Duplicate value';
    if (detail) {
        const matches = detail.match(/(?<=\().+?(?=\))/gm);
        if (matches) {
            errorText = `Duplicate value in the "${matches[0]}"`;
        }
    }
    const error = {
        code: 'invalid_arguments',
        message: detail ? (0, utils_1.formErrorString)(entityMetadata, errorText) : errorText,
        path: ['data', 'attributes'],
    };
    return new common_1.ConflictException([error]);
};
const invalidInputSyntax = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: errorText,
        path: [],
    };
    return new common_1.BadRequestException([error]);
};
const entityHasRelation = (entityMetadata, errorText, detail) => {
    const error = {
        code: 'invalid_arguments',
        message: detail || errorText,
        path: ['data', 'attributes'],
    };
    return new common_1.NotAcceptableException([error]);
};
exports.MysqlError = {
    [1364]: fieldNotNullOrDefault,
    [1062]: duplicateItems,
    [1525]: invalidInputSyntax,
};
exports.PostgresError = {
    [23502]: fieldNotNullOrDefault,
    [23505]: duplicateItems,
    ['22P02']: invalidInputSyntax,
    [22007]: invalidInputSyntax,
    [22003]: invalidInputSyntax,
    [23503]: entityHasRelation,
};
//# sourceMappingURL=index.js.map