"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchOne = patchOne;
const common_1 = require("@nestjs/common");
const utils_1 = require("../../../utils");
async function patchOne(id, inputData) {
    const { id: idBody, attributes, relationships } = inputData;
    if (`${id}` !== idBody) {
        const error = {
            code: 'invalid_arguments',
            message: `Data 'id' must be equal to url param`,
            path: ['data', 'id'],
        };
        throw new common_1.UnprocessableEntityException([error]);
    }
    const paramsId = 'paramsId';
    const result = await this.repository
        .createQueryBuilder()
        .where(`${this.typeormUtilsService.getAliasPath(this.typeormUtilsService.currentPrimaryColumn)} = :${paramsId}`, {
        [paramsId]: id,
    })
        .getOne();
    if (!result) {
        const error = {
            code: 'invalid_arguments',
            message: `Resource '${this.typeormUtilsService.currentAlias}' with id '${id}' does not exist`,
            path: ['data', 'id'],
        };
        throw new common_1.NotFoundException([error]);
    }
    if (attributes) {
        const entityTarget = this.repository.manager.create(this.repository.target, attributes);
        for (const [props, val] of utils_1.ObjectTyped.entries(entityTarget)) {
            result[props] = val;
        }
    }
    const saveData = await this.typeormUtilsService.saveEntityData(result, relationships);
    const { data, included } = this.transformDataService.transformData(saveData);
    const includeData = included ? { included } : {};
    return {
        meta: {},
        data,
        ...includeData,
    };
}
//# sourceMappingURL=patch-one.js.map