import { Repository } from 'typeorm';
import { Type } from '@nestjs/common';
import { CastProps, Concat, Entity, EntityProps, EntityPropsArray, EntityRelation, TypeCast, TypeOfArray, UnionToTuple, ValueOf } from '../../types';
import { ColumnType } from 'typeorm/driver/types/ColumnTypes';
export declare enum PropsNameResultField {
    field = "field",
    relations = "relations"
}
export type ResultGetField<E extends Entity> = {
    [PropsNameResultField.field]: TupleOfEntityProps<E>;
    [PropsNameResultField.relations]: TupleOfEntityRelation<E>;
};
export type TupleOfEntityProps<E, Props = UnionToTuple<EntityProps<E>>> = Props extends readonly [string, ...string[]] ? Props : never;
export type TupleOfEntityRelation<E, Props = UnionToTuple<EntityRelation<E>>> = Props extends readonly [string, ...string[]] ? Props : never;
export type RelationTree<E extends Entity> = {
    [K in keyof RelationType<E>]: TypeOfArray<E[K]> extends Entity ? ResultGetField<TypeOfArray<E[K]>>['field'] : never;
};
export type ConcatFieldWithRelation<R extends string, T extends readonly string[]> = ValueOf<{
    [K in T[number]]: Concat<R, K>;
}>;
export type ConcatRelationUnion<E extends Entity, R = RelationTree<E>> = ValueOf<{
    [K in keyof R]: ConcatFieldWithRelation<TypeCast<K, string>, TypeCast<R[K], readonly string[]>>;
}>;
export type ConcatRelation<E extends Entity> = TypeCast<UnionToTuple<ConcatRelationUnion<E>>, [
    string,
    ...string[]
]>;
type RelationType<E extends Entity> = {
    [K in EntityRelation<E>]: Type<TypeOfArray<CastProps<E, K>>>;
};
export declare enum TypeField {
    array = "array",
    date = "date",
    number = "number",
    boolean = "boolean",
    string = "string",
    object = "object"
}
export type TypeForId = Extract<TypeField, TypeField.number | TypeField.string>;
export type FieldWithType<E extends Entity> = {
    [K in EntityProps<E>]: E[K] extends unknown[] ? TypeField.array : E[K] extends Date ? TypeField.date : E[K] extends number ? TypeField.number : E[K] extends boolean ? TypeField.boolean : E[K] extends object ? TypeField.object : TypeField.string;
};
export type RelationPropsType<E extends Entity> = {
    [K in EntityRelation<E>]: E[K] extends unknown[] ? true : false;
};
export type RelationPropsTypeName<E extends Entity> = {
    [K in EntityRelation<E>]: string;
};
export type RelationPrimaryColumnType<E extends Entity> = {
    [K in EntityRelation<E>]: TypeForId;
};
export declare const getRelationTypeArray: <E extends Entity>(repository: Repository<E>) => RelationPropsType<E>;
export declare const getTypePrimaryColumn: <E extends Entity>(repository: Repository<E>) => TypeForId;
export declare const getRelationTypePrimaryColumn: <E extends Entity>(repository: Repository<E>) => RelationPrimaryColumnType<E>;
export declare const getPrimaryColumnsForRelation: <E extends Entity>(repository: Repository<E>) => RelationPropsTypeName<E>;
export declare const getRelationTypeName: <E extends Entity>(repository: Repository<E>) => RelationPropsTypeName<E>;
export declare const getFieldWithType: <E extends Entity>(repository: Repository<E>) => FieldWithType<E>;
export declare const getField: <E extends Entity>(repository: Repository<E>) => ResultGetField<E>;
export type PropsArray<E> = {
    [K in EntityPropsArray<E>]: true;
};
export declare const getArrayFields: <E extends Entity>(repository: Repository<E>) => PropsArray<E>;
export type ArrayPropsForEntity<E extends Entity> = {
    target: PropsArray<E>;
} & {
    [K in ResultGetField<E>['relations'][number]]: PropsArray<TypeOfArray<CastProps<E, K>>>;
};
export type PropertyTarget<E extends Entity, For extends PropsNameResultField> = {
    [K in ResultGetField<E>[For][number]]: K extends keyof E ? TypeOfArray<E[K]> : never;
};
export declare const getArrayPropsForEntity: <E extends Entity>(repository: Repository<E>) => ArrayPropsForEntity<E>;
export declare const getPropsTreeForRepository: <E extends Entity>(repository: Repository<E>) => RelationTree<E>;
export declare const getIsArrayRelation: <E extends Entity>(repository: Repository<E>) => { [K in EntityRelation<E>]: E[K] extends unknown[] ? true : false; };
export declare const fromRelationTreeToArrayName: <E extends Entity>(relationTree: RelationTree<E>) => ConcatRelation<E>;
export type AllFieldWithTpe<E extends Entity> = FieldWithType<E> & {
    [K in EntityRelation<E>]: E[K] extends (infer U extends Entity)[] ? FieldWithType<U> : E[K] extends Entity ? FieldWithType<E[K]> : never;
};
export declare const getTypeForAllProps: <E extends Entity>(repository: Repository<E>) => AllFieldWithTpe<E>;
export type PropsFieldItem = {
    type: ColumnType;
    isArray: boolean;
    isNullable: boolean;
};
export type PropsForField<E extends Entity> = {
    [K in EntityProps<E>]: PropsFieldItem;
};
export declare const getPropsFromDb: <E extends Entity>(repository: Repository<E>) => PropsForField<E>;
export {};
