"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropsFromDb = exports.getTypeForAllProps = exports.fromRelationTreeToArrayName = exports.getIsArrayRelation = exports.getPropsTreeForRepository = exports.getArrayPropsForEntity = exports.getArrayFields = exports.getField = exports.getFieldWithType = exports.getRelationTypeName = exports.getPrimaryColumnsForRelation = exports.getRelationTypePrimaryColumn = exports.getTypePrimaryColumn = exports.getRelationTypeArray = exports.TypeField = exports.PropsNameResultField = void 0;
const utils_1 = require("../utils");
const orm_type_asserts_1 = require("./orm-type-asserts");
var PropsNameResultField;
(function (PropsNameResultField) {
    PropsNameResultField["field"] = "field";
    PropsNameResultField["relations"] = "relations";
})(PropsNameResultField || (exports.PropsNameResultField = PropsNameResultField = {}));
var TypeField;
(function (TypeField) {
    TypeField["array"] = "array";
    TypeField["date"] = "date";
    TypeField["number"] = "number";
    TypeField["boolean"] = "boolean";
    TypeField["string"] = "string";
    TypeField["object"] = "object";
})(TypeField || (exports.TypeField = TypeField = {}));
const getRelationTypeArray = (repository) => {
    const { relations } = (0, exports.getField)(repository);
    const entity = repository.target;
    const result = {};
    for (const item of relations) {
        result[item] =
            Reflect.getMetadata('design:type', entity['prototype'], item) === Array;
    }
    return result;
};
exports.getRelationTypeArray = getRelationTypeArray;
const getTypePrimaryColumn = (repository) => {
    const target = repository.target;
    const primaryColumn = repository.metadata.primaryColumns[0].propertyName;
    return Reflect.getMetadata('design:type', target['prototype'], primaryColumn) === Number
        ? TypeField.number
        : TypeField.string;
};
exports.getTypePrimaryColumn = getTypePrimaryColumn;
const getRelationTypePrimaryColumn = (repository) => {
    return repository.metadata.relations.reduce((acum, i) => {
        const target = i.inverseEntityMetadata.target;
        const primaryColumn = i.inverseEntityMetadata.primaryColumns[0].propertyName;
        acum[i.propertyName] =
            Reflect.getMetadata('design:type', target['prototype'], primaryColumn) ===
                Number
                ? TypeField.number
                : TypeField.string;
        return acum;
    }, {});
};
exports.getRelationTypePrimaryColumn = getRelationTypePrimaryColumn;
const getPrimaryColumnsForRelation = (repository) => {
    return repository.metadata.relations.reduce((acum, i) => {
        const target = i.inverseEntityMetadata.target;
        acum[i.propertyName] =
            i.inverseEntityMetadata.primaryColumns[0].propertyName;
        return acum;
    }, {});
};
exports.getPrimaryColumnsForRelation = getPrimaryColumnsForRelation;
const getRelationTypeName = (repository) => {
    return repository.metadata.relations.reduce((acum, i) => {
        acum[i.propertyName] = (0, utils_1.getEntityName)(i.inverseEntityMetadata.target);
        return acum;
    }, {});
};
exports.getRelationTypeName = getRelationTypeName;
const getFieldWithType = (repository) => {
    const { field } = (0, exports.getField)(repository);
    const entity = repository.target;
    const result = {};
    for (const item of field) {
        let typeProps = TypeField.string;
        switch (Reflect.getMetadata('design:type', entity['prototype'], item)) {
            case Array:
                typeProps = TypeField.array;
                break;
            case Date:
                typeProps = TypeField.date;
                break;
            case Number:
                typeProps = TypeField.number;
                break;
            case Boolean:
                typeProps = TypeField.boolean;
                break;
            case Object:
                typeProps = TypeField.object;
                break;
            default:
                typeProps = TypeField.string;
        }
        result[item] = typeProps;
    }
    return result;
};
exports.getFieldWithType = getFieldWithType;
const getField = (repository) => {
    const relations = repository.metadata.relations.map((i) => {
        return i.propertyName;
    });
    const field = repository.metadata.columns
        .filter((i) => !relations.includes(i.propertyName))
        .map((r) => r.propertyName);
    return {
        field,
        relations,
    };
};
exports.getField = getField;
const getArrayFields = (repository) => {
    const relations = repository.metadata.relations.map((i) => {
        return i.propertyName;
    });
    return repository.metadata.columns
        .filter((i) => !relations.includes(i.propertyName))
        .reduce((acum, metaData) => {
        if (metaData.isArray) {
            acum[metaData.propertyName] = true;
        }
        return acum;
    }, {});
};
exports.getArrayFields = getArrayFields;
const getArrayPropsForEntity = (repository) => {
    const connection = repository.metadata.connection;
    const relationsTargets = repository.metadata.relations.reduce((acum, i) => ({
        ...acum,
        [i.propertyName]: i.type,
    }), {});
    const { relations } = (0, exports.getField)(repository);
    const relationsArrayFields = relations.reduce((acum, item) => {
        (0, orm_type_asserts_1.guardKeyForPropertyTarget)(relationsTargets, item);
        const target = relationsTargets[item];
        const repository = connection.getRepository(target);
        acum[item] = (0, exports.getArrayFields)(repository);
        return acum;
    }, {});
    return {
        target: (0, exports.getArrayFields)(repository),
        ...relationsArrayFields,
    };
};
exports.getArrayPropsForEntity = getArrayPropsForEntity;
const getPropsTreeForRepository = (repository) => {
    const dataSource = repository.metadata.connection;
    const relationType = repository.metadata.relations.reduce((acum, i) => {
        acum[i.propertyName] = i.inverseEntityMetadata.target;
        return acum;
    }, {});
    return utils_1.ObjectTyped.entries(relationType).reduce((acum, [key, value]) => ({
        ...acum,
        ...{ [key]: (0, exports.getField)(dataSource.getRepository(value))['field'] },
    }), {});
};
exports.getPropsTreeForRepository = getPropsTreeForRepository;
const getIsArrayRelation = (repository) => {
    return repository.metadata.relations.reduce((acum, i) => {
        switch (i.relationType) {
            case 'one-to-many':
            case 'many-to-many':
                acum[i.propertyName] = true;
                break;
            default:
                acum[i.propertyName] = false;
        }
        return acum;
    }, {});
};
exports.getIsArrayRelation = getIsArrayRelation;
const fromRelationTreeToArrayName = (relationTree) => {
    return utils_1.ObjectTyped.entries(relationTree).reduce((acum, [name, filed]) => {
        acum.push(...filed.map((i) => `${name.toLocaleString()}.${i}`));
        return acum;
    }, []);
};
exports.fromRelationTreeToArrayName = fromRelationTreeToArrayName;
const getTypeForAllProps = (repository) => {
    const targetField = (0, exports.getFieldWithType)(repository);
    const relationField = repository.metadata.relations.reduce((acum, item) => {
        acum[item.propertyName] = (0, exports.getFieldWithType)(repository.manager.getRepository(item.inverseEntityMetadata.target));
        return acum;
    }, {});
    return {
        ...targetField,
        ...relationField,
    };
};
exports.getTypeForAllProps = getTypeForAllProps;
const getPropsFromDb = (repository) => {
    return repository.metadata.columns.reduce((acum, i) => {
        acum[i.propertyName] = {
            type: i.type,
            isArray: i.isArray,
            isNullable: i.isNullable,
        };
        return acum;
    }, {});
};
exports.getPropsFromDb = getPropsFromDb;
//# sourceMappingURL=orm-helper.js.map