"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteRelationship = deleteRelationship;
const swagger_1 = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const zod_openapi_1 = require("@anatine/zod-openapi");
const zod_nestjs_1 = require("@anatine/zod-nestjs");
const utils_1 = require("../utils");
const orm_1 = require("../../orm");
const zod_1 = require("../../zod");
function deleteRelationship(controller, repository, binding, config) {
    const entityName = repository.metadata.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller.constructor.prototype, binding.name);
    if (!descriptor)
        throw new Error(`Descriptor for entity controller ${entityName} is empty`);
    const { relations } = (0, orm_1.getField)(repository);
    const classBodySchemaDto = (0, zod_nestjs_1.createZodDto)((0, zod_openapi_1.extendApi)((0, zod_1.zodInputPostSchema)(repository)));
    Object.defineProperty(classBodySchemaDto, 'name', {
        value: `${entityName}DeleteRelationship`,
    });
    (0, swagger_1.ApiExtraModels)(classBodySchemaDto)(controller.constructor);
    (0, swagger_1.ApiOperation)({
        summary: `Delete list of relation for resource "${entityName}"`,
        operationId: `${controller.name}_${binding.name}`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: config.pipeForId === common_1.ParseIntPipe ? 'integer' : 'string',
        description: `ID of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'relName',
        required: true,
        type: 'string',
        enum: relations,
        description: `Relation name of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiBody)({
        description: `Json api schema for delete "${entityName}" item`,
        schema: (0, zod_openapi_1.generateSchema)(zod_1.zodInputPatchRelationshipSchema),
        required: true,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong url parameters',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 422,
        description: 'Incorrect type for relation',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 404,
        description: 'Resource not found ',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 204,
        description: `Item/s of relation for "${entityName}" has been deleted`,
    })(controller, binding.name, descriptor);
}
//# sourceMappingURL=delete-relationship.js.map