"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOne = getOne;
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const utils_1 = require("../utils");
const orm_1 = require("../../orm");
const utils_2 = require("../../utils");
function getOne(controller, repository, binding, config) {
    const entityName = repository.metadata.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller, binding.name);
    if (!descriptor)
        throw new Error(`Descriptor for entity controller ${entityName}:${binding.name} is empty`);
    const { relations, field } = (0, orm_1.getField)(repository);
    const propsTree = (0, orm_1.getPropsTreeForRepository)(repository);
    const relationPrimaryColum = (0, orm_1.getPrimaryColumnsForRelation)(repository);
    const primaryColumn = repository.metadata.primaryColumns[0].propertyName;
    (0, swagger_1.ApiOperation)({
        summary: `Get one item of resource "${entityName}"`,
        operationId: `${controller.constructor.name}_${binding.name}`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: config.pipeForId === common_1.ParseIntPipe ? 'integer' : 'string',
        description: `ID of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'fields',
        required: false,
        style: 'deepObject',
        schema: {
            type: 'object',
        },
        examples: {
            allField: {
                summary: 'Select all field',
                description: 'Select field for target and relation',
                value: {
                    target: field.join(','),
                    ...utils_2.ObjectTyped.entries(propsTree).reduce((acum, [name, props]) => {
                        acum[name.toString()] = props.join(',');
                        return acum;
                    }, {}),
                },
            },
            selectOnlyIdsTarget: {
                summary: 'Select ids for target',
                description: 'Select ids for target',
                value: {
                    target: field.filter((i) => i === primaryColumn).join(','),
                },
            },
            selectOnlyIds: {
                summary: 'Select ids',
                description: 'Select ids',
                value: {
                    target: field.filter((i) => i === primaryColumn).join(','),
                    ...utils_2.ObjectTyped.entries(relationPrimaryColum).reduce((acum, [name, props]) => {
                        acum[name.toString()] = props;
                        return acum;
                    }, {}),
                },
            },
        },
        description: `Object of field for select field from "${entityName}" resource`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'include',
        required: false,
        enum: relations,
        style: 'simple',
        isArray: true,
        description: `"${entityName}" resource item has been extended with existing relations`,
        examples: {
            withInclude: {
                summary: 'Add all relation',
                description: 'Add all realtion',
                value: relations,
            },
            without: {
                summary: 'Without relation',
                description: 'Without all realtion',
                value: [],
            },
        },
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 404,
        description: `Item of resource "${entityName}" not found`,
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong query parameters',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        description: 'Resource one item received successfully',
        schema: (0, utils_1.jsonSchemaResponse)(repository),
    })(controller, binding.name, descriptor);
}
//# sourceMappingURL=get-one.js.map