"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRelationship = getRelationship;
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const utils_1 = require("../utils");
const orm_1 = require("../../orm");
function getRelationship(controller, repository, binding, config) {
    const entityName = repository.metadata.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller, binding.name);
    if (!descriptor)
        throw new Error(`Descriptor for entity controller ${entityName} is empty`);
    const { relations } = (0, orm_1.getField)(repository);
    (0, swagger_1.ApiOperation)({
        summary: `Get list of relation for resource "${entityName}"`,
        operationId: `${controller.constructor.name}_${binding.name}`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: config.pipeForId === common_1.ParseIntPipe ? 'integer' : 'string',
        description: `ID of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'relName',
        required: true,
        type: 'string',
        enum: relations,
        description: `Relation name of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        schema: utils_1.schemaTypeForRelation,
        description: `Item/s of relation for "${entityName}" has been created`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong url parameters',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 422,
        description: 'Incorrect type for relation',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 404,
        description: 'Resource not found ',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
}
//# sourceMappingURL=get-relationship.js.map