"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchOne = patchOne;
const swagger_1 = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const zod_openapi_1 = require("@anatine/zod-openapi");
const utils_1 = require("../utils");
const zod_1 = require("../../zod");
const zod_nestjs_1 = require("@anatine/zod-nestjs");
function patchOne(controller, repository, binding, config) {
    const entityName = repository.metadata.name;
    const descriptor = Reflect.getOwnPropertyDescriptor(controller, binding.name);
    if (!descriptor)
        throw new Error(`Descriptor for entity controller ${entityName} is empty`);
    const classBodySchemaDto = (0, zod_nestjs_1.createZodDto)((0, zod_openapi_1.extendApi)((0, zod_1.zodInputPostSchema)(repository)));
    Object.defineProperty(classBodySchemaDto, 'name', {
        value: `${entityName}PatchOne`,
    });
    (0, swagger_1.ApiExtraModels)(classBodySchemaDto)(controller.constructor);
    (0, swagger_1.ApiOperation)({
        summary: `Update item of resource "${entityName}"`,
        operationId: `${controller.constructor.name}_${binding.name}`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiParam)({
        name: 'id',
        required: true,
        type: config.pipeForId === common_1.ParseIntPipe ? 'integer' : 'string',
        description: `ID of resource "${entityName}"`,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiBody)({
        description: `Json api schema for update "${entityName}" item`,
        schema: (0, zod_openapi_1.generateSchema)((0, zod_1.zodInputPatchSchema)(repository)),
        required: true,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        description: `Item of resource "${entityName}" has been updated`,
        schema: (0, utils_1.jsonSchemaResponse)(repository),
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong body parameters',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 422,
        description: 'Unprocessable data',
        schema: utils_1.errorSchema,
    })(controller, binding.name, descriptor);
}
//# sourceMappingURL=patch-one.js.map