import { SchemaObject } from '@nestjs/swagger/dist/interfaces/open-api-spec.interface';
import { Type } from '@nestjs/common';
import { Repository } from 'typeorm';
import { Entity } from '../../types';
export declare const errorSchema: {
    type: string;
    properties: {
        statusCode: {
            type: string;
        };
        error: {
            type: string;
        };
        message: {
            type: string;
            items: {
                type: string;
                properties: {
                    code: {
                        type: string;
                    };
                    message: {
                        type: string;
                    };
                    path: {
                        type: string;
                        items: {
                            type: string;
                        };
                    };
                    keys: {
                        type: string;
                        items: {
                            type: string;
                        };
                    };
                };
                required: string[];
            };
        };
    };
};
export declare function jsonSchemaResponse<E extends Entity>(repository: Repository<E>, array?: boolean): {
    type: string;
    properties: {
        meta: {
            type: string;
        };
        data: {
            type: string;
            properties: {
                type: {
                    type: string;
                    enum: string[];
                };
                id: {
                    type: string;
                };
                attributes: {
                    type: string;
                    properties: Record<string, SchemaObject>;
                };
                relationships: {
                    type: string;
                    properties: Record<string, SchemaObject>;
                };
                links: {
                    type: string;
                    properties: {
                        self: {
                            type: string;
                        };
                    };
                    required: string[];
                };
            };
        } | {
            type: string;
            items: {
                type: string;
                properties: {
                    type: {
                        type: string;
                        enum: string[];
                    };
                    id: {
                        type: string;
                    };
                    attributes: {
                        type: string;
                        properties: Record<string, SchemaObject>;
                    };
                    relationships: {
                        type: string;
                        properties: Record<string, SchemaObject>;
                    };
                    links: {
                        type: string;
                        properties: {
                            self: {
                                type: string;
                            };
                        };
                        required: string[];
                    };
                };
            };
        };
        includes: {
            type: string;
            items: {
                type: string;
                properties: {
                    type: {
                        type: string;
                    };
                    id: {
                        type: string;
                    };
                    attributes: {
                        type: string;
                    };
                    relationships: {
                        type: string;
                        properties: {
                            relationName: {
                                properties: {
                                    links: {
                                        type: string;
                                        properties: {
                                            self: {
                                                type: string;
                                            };
                                        };
                                        required: string[];
                                    };
                                };
                                required: string[];
                            };
                        };
                    };
                    links: {
                        type: string;
                        properties: {
                            self: {
                                type: string;
                            };
                        };
                        required: string[];
                    };
                };
                required: string[];
            };
        };
    };
    required: string[];
};
export declare function createApiModels<E extends Entity>(repository: Repository<E>): Type<E>;
export declare const schemaTypeForRelation: {
    type: string;
    properties: {
        data: {
            oneOf: ({
                type: string;
                properties: {
                    type: {
                        type: string;
                    };
                    id: {
                        type: string;
                    };
                };
            } | {
                type: string;
                items?: undefined;
            } | {
                type: string;
                items: {
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                        id: {
                            type: string;
                        };
                    };
                };
            })[];
        };
    };
};
