"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.schemaTypeForRelation = exports.errorSchema = void 0;
exports.jsonSchemaResponse = jsonSchemaResponse;
exports.createApiModels = createApiModels;
const swagger_1 = require("@nestjs/swagger");
const orm_1 = require("../orm");
const utils_1 = require("../utils");
exports.errorSchema = {
    type: 'object',
    properties: {
        statusCode: {
            type: 'number',
        },
        error: {
            type: 'string',
        },
        message: {
            type: 'array',
            items: {
                type: 'object',
                properties: {
                    code: {
                        type: 'string',
                    },
                    message: {
                        type: 'string',
                    },
                    path: {
                        type: 'array',
                        items: {
                            type: 'string',
                        },
                    },
                    keys: {
                        type: 'array',
                        items: {
                            type: 'string',
                        },
                    },
                },
                required: ['code', 'message', 'path'],
            },
        },
    },
};
function jsonSchemaResponse(repository, array = false) {
    const { relations } = (0, orm_1.getField)(repository);
    const fieldTypes = (0, orm_1.getFieldWithType)(repository);
    const arrayField = (0, orm_1.getIsArrayRelation)(repository);
    const relationTypeName = (0, orm_1.getRelationTypeName)(repository);
    const primaryColumn = repository.metadata.primaryColumns[0].propertyName;
    const dataType = {
        type: 'object',
        properties: {
            type: {
                type: 'string',
                enum: [(0, utils_1.camelToKebab)(repository.metadata.name)],
            },
            id: {
                type: 'string',
            },
            attributes: {
                type: 'object',
                properties: utils_1.ObjectTyped.entries(fieldTypes)
                    .filter(([name]) => name !== primaryColumn)
                    .reduce((acum, [name, type]) => {
                    switch (type) {
                        case orm_1.TypeField.array:
                            acum[name.toString()] = {
                                type: 'array',
                                items: {
                                    type: 'string',
                                },
                            };
                            break;
                        case orm_1.TypeField.date:
                            acum[name.toString()] = {
                                format: 'date-time',
                                type: 'string',
                            };
                            break;
                        case orm_1.TypeField.number:
                            acum[name.toString()] = {
                                type: 'integer',
                            };
                            break;
                        case orm_1.TypeField.boolean:
                            acum[name.toString()] = {
                                type: 'boolean',
                            };
                            break;
                        default:
                            acum[name.toString()] = {
                                type: 'string',
                            };
                    }
                    return acum;
                }, {}),
            },
            relationships: {
                type: 'object',
                properties: relations.reduce((acum, name) => {
                    const dataItem = {
                        type: 'object',
                        properties: {
                            type: {
                                type: 'string',
                                enum: [
                                    (0, utils_1.camelToKebab)(relationTypeName[name]),
                                ],
                            },
                            id: {
                                type: 'string',
                            },
                        },
                        required: ['type', 'id'],
                    };
                    const dataArray = {
                        type: 'array',
                        items: dataItem,
                    };
                    acum[name.toString()] = {
                        type: 'object',
                        properties: {
                            links: {
                                type: 'object',
                                properties: {
                                    self: {
                                        type: 'string',
                                    },
                                },
                                required: ['self'],
                            },
                            data: arrayField[name]
                                ? dataArray
                                : dataItem,
                        },
                        required: ['links'],
                    };
                    return acum;
                }, {}),
            },
            links: {
                type: 'object',
                properties: {
                    self: {
                        type: 'string',
                    },
                },
                required: ['self'],
            },
        },
    };
    const dataTypeArra = {
        type: 'array',
        items: dataType,
    };
    return {
        type: 'object',
        properties: {
            meta: {
                type: 'object',
            },
            data: array ? dataTypeArra : dataType,
            includes: {
                type: 'array',
                items: {
                    type: 'object',
                    properties: {
                        type: {
                            type: 'string',
                        },
                        id: {
                            type: 'string',
                        },
                        attributes: {
                            type: 'object',
                        },
                        relationships: {
                            type: 'object',
                            properties: {
                                relationName: {
                                    properties: {
                                        links: {
                                            type: 'object',
                                            properties: {
                                                self: {
                                                    type: 'string',
                                                },
                                            },
                                            required: ['self'],
                                        },
                                    },
                                    required: ['links'],
                                },
                            },
                        },
                        links: {
                            type: 'object',
                            properties: {
                                self: {
                                    type: 'string',
                                },
                            },
                            required: ['self'],
                        },
                    },
                    required: ['type', 'id', 'attributes'],
                },
            },
        },
        required: ['meta', 'data'],
    };
}
function createApiModels(repository) {
    const propsType = (0, orm_1.getFieldWithType)(repository);
    const relationTypeName = (0, orm_1.getRelationTypeName)(repository);
    const relationArray = (0, orm_1.getIsArrayRelation)(repository);
    const result = repository.create({
        ...propsType,
        ...utils_1.ObjectTyped.entries(relationTypeName).reduce((acum, [name, value]) => {
            acum[name.toString()] = relationArray[name] ? [value] : value;
            return acum;
        }, {}),
    });
    const newEntity = (0, utils_1.nameIt)(repository.metadata.name, class {
    });
    for (const [name, value] of Object.entries(result)) {
        let currentType;
        let isArray = false;
        if (name in propsType) {
            switch (value) {
                case orm_1.TypeField.array:
                    currentType = String;
                    isArray = true;
                    break;
                case orm_1.TypeField.date:
                    currentType = Date;
                    break;
                case orm_1.TypeField.number:
                    currentType = Number;
                    break;
                case orm_1.TypeField.boolean:
                    currentType = Boolean;
                    break;
                default:
                    currentType = String;
            }
        }
        else {
            currentType = relationTypeName[name];
            isArray = Array.isArray(value);
        }
        (0, swagger_1.ApiProperty)({
            required: false,
            isArray: isArray,
            type: () => currentType,
        })(newEntity.prototype, name);
    }
    return newEntity;
}
const dataType = {
    type: 'object',
    properties: {
        type: {
            type: 'string',
        },
        id: {
            type: 'string',
        },
    },
};
exports.schemaTypeForRelation = {
    type: 'object',
    properties: {
        data: {
            oneOf: [
                dataType,
                { type: 'null' },
                {
                    type: 'array',
                    items: dataType,
                },
            ],
        },
    },
};
//# sourceMappingURL=utils.js.map