import { Repository } from 'typeorm';
import { z, ZodObject, ZodUnion } from 'zod';
import { QueryField } from "../../../../json-shared-type/src";
import { Entity } from '../../types';
import { ZodInputQueryShape } from './zod-input-query-schema';
import { ZodQueryShape } from './zod-query-schema';
import { PostShape } from './zod-input-post-schema';
import { PatchShape, PatchShapeDefault } from './zod-input-patch-schema';
import { PostRelationshipSchema } from './zod-input-post-relationship-schema';
import { PatchRelationshipSchema } from './zod-input-patch-relationship-schema';
export { QueryField };
export type ZodInputQuerySchema<E extends Entity> = ZodObject<ZodInputQueryShape<E>, 'strict'>;
export type InputQuery<E extends Entity> = z.infer<ZodInputQuerySchema<E>>;
export type ZodQuerySchema<E extends Entity> = ZodObject<ZodQueryShape<E>, 'strict'>;
export type Query<E extends Entity> = z.infer<ZodQuerySchema<E>>;
export type TypeInputProps<E extends Entity, K extends keyof ZodInputQueryShape<E>> = z.infer<ZodInputQueryShape<E>[K]>;
export type ZodInputPostSchema<E extends Entity> = ZodObject<{
    data: ZodObject<PostShape<E>, 'strict'>;
}, 'strict'>;
export type PostData<E extends Entity> = z.infer<ZodInputPostSchema<E>>['data'];
export type ZodInputPatchSchema<E extends Entity> = ZodObject<{
    data: ZodUnion<[
        ZodObject<PatchShapeDefault<E>, 'strict'>,
        ZodObject<PatchShape<E>, 'strict'>
    ]>;
}, 'strict'>;
export type PatchData<E extends Entity> = z.infer<ZodInputPatchSchema<E>>['data'];
export type ZodInputPostRelationshipSchema = ZodObject<{
    data: PostRelationshipSchema;
}, 'strict'>;
export type PostRelationshipData = z.infer<ZodInputPostRelationshipSchema>['data'];
export type ZodInputPatchRelationshipSchema = ZodObject<{
    data: PatchRelationshipSchema;
}, 'strict'>;
export type PatchRelationshipData = z.infer<ZodInputPatchRelationshipSchema>['data'];
export declare const zodInputQuerySchema: <E extends Entity>(repository: Repository<E>) => ZodInputQuerySchema<E>;
export declare const zodQuerySchema: <E extends Entity>(repository: Repository<E>) => ZodQuerySchema<E>;
export declare const zodInputPostSchema: <E extends Entity>(repository: Repository<E>) => ZodInputPostSchema<E>;
export declare const zodInputPatchSchema: <E extends Entity>(repository: Repository<E>) => ZodInputPatchSchema<E>;
export declare const zodInputPostRelationshipSchema: ZodInputPostRelationshipSchema;
export declare const zodInputPatchRelationshipSchema: ZodInputPatchRelationshipSchema;
