"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodInputPatchRelationshipSchema = exports.zodInputPostRelationshipSchema = exports.zodInputPatchSchema = exports.zodInputPostSchema = exports.zodQuerySchema = exports.zodInputQuerySchema = exports.QueryField = void 0;
const zod_1 = require("zod");
const json_shared_type_1 = require("../../../../json-shared-type/src");
Object.defineProperty(exports, "QueryField", { enumerable: true, get: function () { return json_shared_type_1.QueryField; } });
const orm_1 = require("../orm");
const zod_input_query_schema_1 = require("./zod-input-query-schema");
const zod_query_schema_1 = require("./zod-query-schema");
const zod_input_post_schema_1 = require("./zod-input-post-schema");
const zod_input_patch_schema_1 = require("./zod-input-patch-schema");
const zod_input_post_relationship_schema_1 = require("./zod-input-post-relationship-schema");
const zod_input_patch_relationship_schema_1 = require("./zod-input-patch-relationship-schema");
const utils_1 = require("../utils");
const id_1 = require("./zod-input-post-schema/id");
const zodInputQuerySchema = (repository) => {
    const { field, relations } = (0, orm_1.getField)(repository);
    const relationTree = (0, orm_1.fromRelationTreeToArrayName)((0, orm_1.getPropsTreeForRepository)(repository));
    const zodInputQueryShape = {
        [json_shared_type_1.QueryField.filter]: (0, zod_input_query_schema_1.zodFilterInputQuerySchema)(field, relations, relationTree).optional(),
        [json_shared_type_1.QueryField.fields]: (0, zod_input_query_schema_1.zodSelectFieldsInputQuerySchema)(relations).optional(),
        [json_shared_type_1.QueryField.include]: zod_input_query_schema_1.zodIncludeInputQuerySchema.optional(),
        [json_shared_type_1.QueryField.sort]: zod_input_query_schema_1.zodSortInputQuerySchema.optional(),
        [json_shared_type_1.QueryField.page]: zod_input_query_schema_1.zodPageInputQuerySchema,
    };
    return zod_1.z
        .object(zodInputQueryShape)
        .strict(`Query object should contain only allow params: "${Object.keys(json_shared_type_1.QueryField).join('"."')}"`);
};
exports.zodInputQuerySchema = zodInputQuerySchema;
const zodQuerySchema = (repository) => {
    const { field, relations } = (0, orm_1.getField)(repository);
    const relationTree = (0, orm_1.getPropsTreeForRepository)(repository);
    const propsArray = (0, orm_1.getArrayPropsForEntity)(repository);
    const typeProps = (0, orm_1.getTypeForAllProps)(repository);
    const zodQueryShape = {
        [json_shared_type_1.QueryField.filter]: (0, zod_query_schema_1.zodFilterQuerySchema)(field, relationTree, propsArray, typeProps),
        [json_shared_type_1.QueryField.fields]: (0, zod_query_schema_1.zodSelectFieldsQuerySchema)(field, relationTree).nullable(),
        [json_shared_type_1.QueryField.include]: (0, zod_query_schema_1.zodIncludeQuerySchema)(relations).nullable(),
        [json_shared_type_1.QueryField.sort]: (0, zod_query_schema_1.zodSortQuerySchema)(field, relationTree).nullable(),
        [json_shared_type_1.QueryField.page]: zod_query_schema_1.zodPageQuerySchema,
    };
    return zod_1.z.object(zodQueryShape).strict();
};
exports.zodQuerySchema = zodQuerySchema;
const zodInputPostSchema = (repository) => {
    const relationArrayProps = (0, orm_1.getRelationTypeArray)(repository);
    const relationPopsName = (0, orm_1.getRelationTypeName)(repository);
    const primaryColumnType = (0, orm_1.getRelationTypePrimaryColumn)(repository);
    const primaryType = (0, orm_1.getTypePrimaryColumn)(repository);
    const fieldWithType = utils_1.ObjectTyped.entries((0, orm_1.getFieldWithType)(repository))
        .filter(([key]) => key !== repository.metadata.primaryColumns[0].propertyName)
        .reduce((acum, [key, type]) => ({
        ...acum,
        [key]: type,
    }), {});
    const typeName = (0, utils_1.camelToKebab)((0, utils_1.getEntityName)(repository.target));
    const propsDb = (0, orm_1.getPropsFromDb)(repository);
    const postShape = {
        id: (0, id_1.zodIdSchema)(primaryType).optional(),
        type: (0, zod_input_post_schema_1.zodTypeSchema)(typeName),
        attributes: (0, zod_input_post_schema_1.zodAttributesSchema)(fieldWithType, propsDb),
        relationships: (0, zod_input_post_schema_1.zodRelationshipsSchema)(relationArrayProps, relationPopsName, primaryColumnType).optional(),
    };
    return zod_1.z
        .object({
        data: zod_1.z.object(postShape).strict(),
    })
        .strict();
};
exports.zodInputPostSchema = zodInputPostSchema;
const zodInputPatchSchema = (repository) => {
    const relationArrayProps = (0, orm_1.getRelationTypeArray)(repository);
    const relationPopsName = (0, orm_1.getRelationTypeName)(repository);
    const primaryColumnType = (0, orm_1.getRelationTypePrimaryColumn)(repository);
    const primaryType = (0, orm_1.getTypePrimaryColumn)(repository);
    const fieldWithType = utils_1.ObjectTyped.entries((0, orm_1.getFieldWithType)(repository))
        .filter(([key]) => key !== repository.metadata.primaryColumns[0].propertyName)
        .reduce((acum, [key, type]) => ({
        ...acum,
        [key]: type,
    }), {});
    const typeName = (0, utils_1.camelToKebab)((0, utils_1.getEntityName)(repository.target));
    const propsDb = (0, orm_1.getPropsFromDb)(repository);
    const patchShapeDefault = {
        id: (0, id_1.zodIdSchema)(primaryType),
        type: (0, zod_input_post_schema_1.zodTypeSchema)(typeName),
        attributes: (0, zod_input_post_schema_1.zodAttributesSchema)(fieldWithType, propsDb),
        relationships: (0, zod_input_patch_schema_1.zodPatchRelationshipsSchema)(relationArrayProps, relationPopsName, primaryColumnType).optional(),
    };
    const patchShape = {
        id: (0, id_1.zodIdSchema)(primaryType),
        type: (0, zod_input_post_schema_1.zodTypeSchema)(typeName),
        attributes: (0, zod_input_post_schema_1.zodAttributesSchema)(fieldWithType, propsDb)
            .optional()
            .default({}),
        relationships: (0, zod_input_patch_schema_1.zodPatchRelationshipsSchema)(relationArrayProps, relationPopsName, primaryColumnType),
    };
    return zod_1.z
        .object({
        data: zod_1.z.union([
            zod_1.z.object(patchShapeDefault).strict(),
            zod_1.z.object(patchShape).strict(),
        ]),
    })
        .strict();
};
exports.zodInputPatchSchema = zodInputPatchSchema;
exports.zodInputPostRelationshipSchema = zod_1.z
    .object({
    data: zod_input_post_relationship_schema_1.postRelationshipSchema,
})
    .strict();
exports.zodInputPatchRelationshipSchema = zod_1.z
    .object({
    data: zod_input_patch_relationship_schema_1.patchRelationshipSchema,
})
    .strict();
//# sourceMappingURL=zod-helper.js.map