import { ZodArray, ZodBoolean, ZodEnum, ZodNumber, ZodString, ZodDate, ZodObject, ZodEffects, ZodOptional, ZodNullable } from 'zod';
import { Entity } from '../../../types';
import { FieldWithType, PropsForField, TypeField } from '../../orm';
type ZodTypeForArray = ZodString | ZodDate | ZodEffects<ZodNumber, number, unknown> | ZodBoolean;
type ZodArrayType = ZodArray<ZodTypeForArray, 'many'> | ZodNullable<ZodArray<ZodTypeForArray, 'many'>>;
type TypeMapToZod = {
    [TypeField.array]: ZodOptional<ZodArrayType>;
    [TypeField.date]: ZodOptional<ZodDate>;
    [TypeField.number]: ZodOptional<ZodNumber>;
    [TypeField.boolean]: ZodOptional<ZodBoolean>;
    [TypeField.string]: ZodOptional<ZodString | ZodEnum<[string, ...string[]]>>;
    [TypeField.object]: ZodObject<{}, 'passthrough'>;
};
type ZodShapeAttributes<E extends Entity> = Omit<{
    [K in keyof FieldWithType<E>]: TypeMapToZod[FieldWithType<E>[K]];
}, 'id'>;
export type ZodAttributesSchema<E extends Entity> = ZodEffects<ZodObject<ZodShapeAttributes<E>, 'strict'>>;
export declare const zodAttributesSchema: <E extends Entity>(fieldWithType: FieldWithType<E>, propsDb: PropsForField<E>) => ZodAttributesSchema<E>;
export {};
