"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodAttributesSchema = void 0;
const zod_1 = require("zod");
const orm_1 = require("../../orm");
const utils_1 = require("../../utils");
const zod_utils_1 = require("../zod-utils");
function getZodSchemaForArray(props) {
    if (!props)
        return zod_1.z.string();
    let zodSchema;
    switch (props.type) {
        case 'number':
        case 'real':
        case Number:
            zodSchema = zod_1.z.preprocess((x) => Number(x), zod_1.z.number());
            break;
        case 'date':
        case Date:
            zodSchema = zod_1.z.coerce.date();
            break;
        case 'boolean':
        case Boolean:
            zodSchema = zod_1.z.boolean();
            break;
        default:
            zodSchema = zod_1.z.string();
    }
    return zodSchema;
}
const zodAttributesSchema = (fieldWithType, propsDb) => {
    const shape = utils_1.ObjectTyped.entries(fieldWithType).reduce((acum, [props, type]) => {
        let zodShema;
        const propsDbType = propsDb[props];
        switch (type) {
            case orm_1.TypeField.array: {
                const tmpSchema = getZodSchemaForArray(propsDbType).array();
                zodShema = (propsDbType && propsDbType.isNullable
                    ? tmpSchema.nullable()
                    : tmpSchema).optional();
                break;
            }
            case orm_1.TypeField.date:
                zodShema = zod_1.z.coerce.date().optional();
                break;
            case orm_1.TypeField.number:
                zodShema = zod_1.z.number().optional();
                break;
            case orm_1.TypeField.boolean:
                zodShema = zod_1.z.boolean().optional();
                break;
            case orm_1.TypeField.object:
                zodShema = zod_1.z.object({}).passthrough();
                break;
            case orm_1.TypeField.string:
                zodShema = zod_1.z.string().optional();
                break;
        }
        return {
            ...acum,
            [props]: zodShema,
        };
    }, {});
    return zod_1.z.object(shape).strict().refine(zod_utils_1.nonEmptyObject);
};
exports.zodAttributesSchema = zodAttributesSchema;
//# sourceMappingURL=attributes.js.map