import { ZodEffects, ZodLiteral, ZodObject, ZodOptional, ZodString, ZodUnion } from 'zod';
import { ZodFilterMap } from './select';
import { Entity, FilterOperand, ValueOf } from '../../../types';
import { ConcatRelation, ResultGetField } from '../../orm';
export type ZodFilterSchema = ZodEffects<ZodObject<{
    [key in ValueOf<typeof FilterOperand>]: ZodOptional<ZodEffects<ZodString>>;
}, 'strict'>>;
export declare const zodFilterSchema: ZodFilterSchema;
export type ZodFilterFieldSchema = ZodUnion<[
    ZodEffects<ZodString>,
    ZodFilterSchema
]>;
export declare const zodFilterFieldSchema: ZodFilterFieldSchema;
export type ZodFilterRelationSchema = ZodUnion<[
    ZodObject<{
        [FilterOperand.eq]: ZodLiteral<'null'>;
    }>,
    ZodObject<{
        [FilterOperand.ne]: ZodLiteral<'null'>;
    }>
]>;
export declare const zodFilterRelationSchema: ZodFilterRelationSchema;
export type ZodFilterInputMap<E extends Entity> = ZodFilterMap<ResultGetField<E>['field'], ZodFilterFieldSchema> & ZodFilterMap<ResultGetField<E>['relations'], ZodFilterRelationSchema> & ZodFilterMap<ConcatRelation<E>, ZodFilterFieldSchema>;
export declare const getObjectForFilter: <E extends readonly [string, ...string[]], R extends boolean>(fieldList: E, isRelation: R) => ZodFilterMap<E, R extends true ? ZodFilterRelationSchema : ZodFilterFieldSchema>;
export type ZodFilterInputQueryObject<E extends Entity> = ZodObject<ZodFilterInputMap<E>, 'strict'>;
export type ZodFilterInputQuerySchema<E extends Entity> = ZodEffects<ZodFilterInputQueryObject<E>>;
export declare const zodFilterInputQuerySchema: <E extends Entity>(field: ResultGetField<E>["field"], relations: ResultGetField<E>["relations"], relationField: ConcatRelation<E>) => ZodFilterInputQuerySchema<E>;
