"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodFilterInputQuerySchema = exports.getObjectForFilter = exports.zodFilterRelationSchema = exports.zodFilterFieldSchema = exports.zodFilterSchema = void 0;
const zod_1 = require("zod");
const types_1 = require("../../../types");
const utils_1 = require("../../utils");
const zod_utils_1 = require("../zod-utils");
exports.zodFilterSchema = zod_1.z
    .object(utils_1.ObjectTyped.values(types_1.FilterOperand).reduce((acum, item) => {
    acum[item] = zod_1.z.string().refine((0, zod_utils_1.stringLongerThan)());
    return acum;
}, {}))
    .partial()
    .strict()
    .refine((0, zod_utils_1.oneOf)(Object.values(types_1.FilterOperand)), {
    message: `Must have one of: "${Object.values(types_1.FilterOperand).join('","')}"`,
});
exports.zodFilterFieldSchema = zod_1.z.union([
    zod_1.z.string().refine((0, zod_utils_1.stringLongerThan)()),
    exports.zodFilterSchema,
]);
exports.zodFilterRelationSchema = zod_1.z.union([
    zod_1.z
        .object({
        [types_1.FilterOperand.eq]: zod_1.z.literal('null'),
    })
        .strict(),
    zod_1.z
        .object({
        [types_1.FilterOperand.ne]: zod_1.z.literal('null'),
    })
        .strict(),
]);
const getObjectForFilter = (fieldList, isRelation) => {
    return fieldList.reduce((acum, item) => ({
        ...acum,
        ...{
            [item]: isRelation
                ? exports.zodFilterRelationSchema.optional()
                : exports.zodFilterFieldSchema.optional(),
        },
    }), {});
};
exports.getObjectForFilter = getObjectForFilter;
const zodFilterInputQuerySchema = (field, relations, relationField) => {
    const filterMap = {
        ...(0, exports.getObjectForFilter)(field, false),
        ...(0, exports.getObjectForFilter)(relations, true),
        ...(0, exports.getObjectForFilter)(relationField, false),
    };
    const zodFilterInputQueryObject = zod_1.z
        .object(filterMap)
        .strict((0, zod_utils_1.getValidationErrorForStrict)(Object.keys(filterMap), 'Filter'));
    return zodFilterInputQueryObject.refine((0, zod_utils_1.nonEmptyObject)(), {
        message: 'Validation error: Filter should be not empty',
    });
};
exports.zodFilterInputQuerySchema = zodFilterInputQuerySchema;
//# sourceMappingURL=filter.js.map