import { ZodArray, ZodEffects, ZodLiteral, ZodNullable, ZodObject, ZodOptional, ZodString, ZodUnion } from 'zod';
import { CastProps, Entity, FilterOperand, IsArray, TypeCast, TypeOfArray, ValueOf } from '../../../types';
import { AllFieldWithTpe, ArrayPropsForEntity, RelationTree, ResultGetField } from '../../orm';
import { ZodFilterRelationSchema } from '../zod-input-query-schema/filter';
type ZodForString = ZodUnion<[
    ZodEffects<ZodLiteral<'null'>, null, 'null'>,
    ZodEffects<ZodString>
]>;
type ZodForStringArray = ZodEffects<ZodArray<ZodForString, 'atleastone'>, [
    string | null,
    ...(string | null)[]
], [
    string,
    ...string[]
]>;
type OperandForString = Exclude<ValueOf<typeof FilterOperand>, FilterOperand.in | FilterOperand.nin | FilterOperand.some>;
type OperandForArray = Extract<ValueOf<typeof FilterOperand>, FilterOperand.in | FilterOperand.nin>;
type OperandForArrayField = FilterOperand.some;
type MapOperandForString = {
    [K in OperandForString]: ZodOptional<ZodForString>;
};
type MapOperandForArrayString = {
    [K in OperandForArray]: ZodOptional<ZodForStringArray>;
};
type MapOperand = MapOperandForString & MapOperandForArrayString;
type ZodOperand = ZodEffects<ZodObject<MapOperand, 'strict'>>;
type ZodOperandForArrayField = ZodObject<{
    [K in OperandForArrayField]: ZodForStringArray;
}, 'strict'>;
type FilterProps<E extends Entity, P extends readonly [string, ...string[]]> = {
    [Props in P[number]]: Props extends keyof E ? IsArray<E[Props]> extends true ? ZodOptional<ZodOperandForArrayField> : ZodOptional<ZodOperand> : never;
};
type FilterTargetRelation<P extends readonly [string, ...string[]]> = {
    [Props in P[number]]: ZodOptional<ZodFilterRelationSchema>;
};
type TargetRelation<E extends Entity> = FilterTargetRelation<ResultGetField<E>['relations']>;
type TargetProps<E extends Entity> = FilterProps<E, ResultGetField<E>['field']> & TargetRelation<E>;
type Target<E extends Entity> = {
    target: ZodNullable<ZodEffects<ZodObject<TargetProps<E>, 'strict'>>>;
};
type RelationType<E extends Entity, R> = TypeCast<TypeOfArray<CastProps<E, R>>, Entity>;
type RelationFilter<E extends Entity> = ZodOptional<ZodEffects<ZodObject<FilterProps<E, ResultGetField<E>['field']>, 'strict'>>>;
type Relation<E extends Entity> = {
    [R in keyof RelationTree<E>]: RelationFilter<RelationType<E, R>>;
};
type ZodObjectRelation<E extends Entity> = ZodNullable<ZodEffects<ZodObject<Relation<E>, 'strict'>>>;
type ShapeFilter<E extends Entity> = Target<E> & {
    relation: ZodObjectRelation<E>;
};
export type ZodFilterQuerySchema<E extends Entity> = ZodEffects<ZodObject<ShapeFilter<E>>>;
export declare const zodFilterQuerySchema: <E extends Entity>(field: ResultGetField<E>["field"], relationTree: RelationTree<E>, propsArray: ArrayPropsForEntity<E>, propsType: AllFieldWithTpe<E>) => ZodFilterQuerySchema<E>;
export {};
