"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodFilterQuerySchema = void 0;
const zod_1 = require("zod");
const zod_utils_1 = require("../zod-utils");
const types_1 = require("../../../types");
const utils_1 = require("../../utils");
const orm_1 = require("../../orm");
const filter_1 = require("../zod-input-query-schema/filter");
const zodForString = zod_1.z.union([
    zod_1.z.literal('null').transform(() => null),
    zod_1.z.string().refine((0, zod_utils_1.stringLongerThan)(), {
        message: 'String should be not empty',
    }),
]);
const zodForStringArray = zodForString
    .array()
    .nonempty()
    .refine((0, zod_utils_1.arrayItemStringLongerThan)(0), {
    message: 'Array should be not empty',
});
const arrayOperand = {
    [types_1.FilterOperand.nin]: true,
    [types_1.FilterOperand.in]: true,
};
const shapeMapOperand = (type = orm_1.TypeField.string) => zod_1.z
    .object(utils_1.ObjectTyped.entries(types_1.FilterOperand)
    .filter(([key]) => key !== types_1.FilterOperand.some)
    .reduce((acum, [key, val]) => ({
    ...acum,
    [val]: (Object.prototype.hasOwnProperty.call(arrayOperand, key)
        ? zodForStringArray.refine((0, zod_utils_1.elementOfArrayMustBe)(type), {
            message: `String should be as ${type}`,
        })
        : zodForString.refine((0, zod_utils_1.stringMustBe)(type), {
            message: `String should be as ${type}`,
        })).optional(),
}), {}))
    .strict()
    .refine((0, zod_utils_1.oneOf)(Object.values(types_1.FilterOperand).filter((i) => i !== types_1.FilterOperand.some)), {
    message: `Must have one of: "${Object.values(types_1.FilterOperand)
        .filter((i) => i !== types_1.FilterOperand.some)
        .join('","')}"`,
});
const shapeForArrayField = zod_1.z
    .object({ [types_1.FilterOperand.some]: zodForStringArray })
    .strict();
const getFilterProps = (field, propsArray, propsType) => field.reduce((acum, item) => ({
    ...acum,
    [item]: (Reflect.get(propsArray, item)
        ? shapeForArrayField
        : shapeMapOperand(propsType[item])).optional(),
}), {});
const zodFilterQuerySchema = (field, relationTree, propsArray, propsType) => {
    const { target: propsArrayTarget, ...relationPropsArray } = propsArray;
    const targetShape = getFilterProps(field, propsArrayTarget, propsType);
    const targetRelation = utils_1.ObjectTyped.keys(relationTree).reduce((acum, item) => ({
        ...acum,
        [item]: filter_1.zodFilterRelationSchema.optional(),
    }), {});
    const targetProps = {
        ...targetShape,
        ...targetRelation,
    };
    const target = {
        target: zod_1.z.object(targetProps).strict().refine((0, zod_utils_1.nonEmptyObject)()).nullable(),
    };
    const relationPlaceHolder = utils_1.ObjectTyped.keys(relationTree).reduce((acum, item) => {
        acum[item] = undefined;
        return acum;
    }, {});
    const relation = utils_1.ObjectTyped.keys(relationTree).reduce((acum, name) => {
        const relationField = relationTree[name];
        (0, orm_1.guardIsKeyOfObject)(relationPropsArray, name);
        const propsArrayForRelation = relationPropsArray[name];
        const filterProps = getFilterProps(relationField, propsArrayForRelation, propsType[name]);
        const zodFilter = zod_1.z
            .object(filterProps)
            .strict()
            .refine((0, zod_utils_1.nonEmptyObject)())
            .optional();
        const newName = name;
        (0, orm_1.guardIsKeyOfObject)(relationPlaceHolder, newName);
        acum[newName] = zodFilter;
        return acum;
    }, {});
    const zodObjectRelation = zod_1.z
        .object(relation)
        .strict()
        .refine((0, zod_utils_1.nonEmptyObject)())
        .nullable();
    const shapeFilter = {
        ...target,
        ['relation']: zodObjectRelation,
    };
    return zod_1.z.object(shapeFilter).strict().refine((0, zod_utils_1.nonEmptyObject)());
};
exports.zodFilterQuerySchema = zodFilterQuerySchema;
//# sourceMappingURL=filter.js.map