import { Entity } from '../../../types';
import { ZodEffects, ZodEnum, ZodObject, ZodOptional } from 'zod';
import { RelationTree, ResultGetField } from '../../orm';
export type SortTarget<F extends readonly [string, ...string[]]> = {
    target: SortEntityOptionalObject<F>;
};
export declare const sortTarget: <R extends readonly [string, ...string[]]>(fields: R) => SortTarget<R>;
export declare const sortEntity: <R extends readonly [string, ...string[]]>(fields: R) => SortEntityOptional<R>;
export type SortEnum = ZodEnum<['DESC', 'ASC']>;
export type SortEntityOptional<F extends readonly [string, ...string[]]> = {
    [K in F[number]]: ZodOptional<SortEnum>;
};
export type SortEntityOptionalObject<F extends readonly [string, ...string[]]> = ZodEffects<ZodObject<SortEntityOptional<F>, 'strict'>>;
export type SortRelation<E extends Entity> = {
    [k in keyof RelationTree<E>]: SortEntityOptionalObject<RelationTree<E>[k]>;
};
export type Sort<E extends Entity> = SortTarget<ResultGetField<E>['field']> & SortRelation<E>;
export type OptionalSort<E extends Entity> = {
    [k in keyof Sort<E>]: ZodOptional<Sort<E>[k]>;
};
type OptionalZodBaseObjectSchema<E extends Entity> = ZodObject<OptionalSort<E>, 'strict'>;
export type ZodSortQuerySchema<E extends Entity> = ZodEffects<OptionalZodBaseObjectSchema<E>>;
export declare const zodSortQuerySchema: <E extends Entity>(fields: ResultGetField<E>["field"], relation: RelationTree<E>) => ZodSortQuerySchema<E>;
export {};
