"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodSortQuerySchema = exports.sortEntity = exports.sortTarget = void 0;
const zod_1 = require("zod");
const constants_1 = require("../../../constants");
const zod_utils_1 = require("../zod-utils");
const utils_1 = require("../../utils");
const sortTarget = (fields) => ({
    target: zod_1.z.object((0, exports.sortEntity)(fields)).strict().refine((0, zod_utils_1.nonEmptyObject)()),
});
exports.sortTarget = sortTarget;
const sortEntity = (fields) => fields.reduce((acum, item) => ({
    ...acum,
    ...{
        [item]: zod_1.z.enum(constants_1.SORT_TYPE).optional(),
    },
}), {});
exports.sortEntity = sortEntity;
const zodSortQuerySchema = (fields, relation) => {
    const sortTargetObject = (0, exports.sortTarget)(fields);
    const sortRelationObject = utils_1.ObjectTyped.entries(relation).reduce((acum, [item, fields]) => {
        return {
            ...acum,
            ...{
                [item]: zod_1.z
                    .object((0, exports.sortEntity)(fields))
                    .strict()
                    .refine((0, zod_utils_1.nonEmptyObject)()),
            },
        };
    }, {});
    const sortMerge = {
        ...sortTargetObject,
        ...sortRelationObject,
    };
    const baseZodSchema = zod_1.z.object(sortMerge).strict();
    const partialZod = baseZodSchema.partial();
    return partialZod.refine((0, zod_utils_1.nonEmptyObject)());
};
exports.zodSortQuerySchema = zodSortQuerySchema;
//# sourceMappingURL=sort.js.map