"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValidationErrorForStrict = exports.nonEmptyObject = exports.uniqueArray = exports.elementOfArrayMustBe = exports.stringMustBe = exports.arrayItemStringLongerThan = exports.stringLongerThan = exports.oneOf = void 0;
const orm_1 = require("../orm");
const oneOf = (keys) => (val) => {
    for (const k of keys) {
        if (val[k] !== undefined)
            return true;
    }
    return false;
};
exports.oneOf = oneOf;
const stringLongerThan = (length = 0) => (str) => str.length > length;
exports.stringLongerThan = stringLongerThan;
const arrayItemStringLongerThan = (length = 0) => (array) => {
    const checkFunction = (0, exports.stringLongerThan)(length);
    return !array.some((i) => i !== null && !checkFunction(i));
};
exports.arrayItemStringLongerThan = arrayItemStringLongerThan;
const stringMustBe = (type = orm_1.TypeField.string) => (inputString) => {
    if (inputString === null)
        return true;
    switch (type) {
        case orm_1.TypeField.boolean:
            return inputString === 'true' || inputString === 'false';
        case orm_1.TypeField.number:
            return !isNaN(+inputString);
        case orm_1.TypeField.date:
            return new Date(inputString).toString() !== 'Invalid Date';
        default:
            return true;
    }
};
exports.stringMustBe = stringMustBe;
const elementOfArrayMustBe = (type = orm_1.TypeField.string) => (inputArray) => {
    const checkFunc = (0, exports.stringMustBe)(type);
    return !inputArray.some((i) => !checkFunc(`${i}`));
};
exports.elementOfArrayMustBe = elementOfArrayMustBe;
const uniqueArray = () => (e) => new Set(e).size === e.length;
exports.uniqueArray = uniqueArray;
const nonEmptyObject = () => (object) => !!(object && Object.keys(object).length > 0);
exports.nonEmptyObject = nonEmptyObject;
const getValidationErrorForStrict = (props, name) => `Validation error: ${name} should be have only props: ["${props.join('","')}"]`;
exports.getValidationErrorForStrict = getValidationErrorForStrict;
//# sourceMappingURL=zod-utils.js.map