import { MethodName, Entity, EntityRelation, ResourceObject, ResourceObjectRelationships } from '../../types';
import { PostData, Query, PatchData, PostRelationshipData, PatchRelationshipData } from '../../helper';
import { TYPEORM_SERVICE_PROPS } from '../../constants';
type RequestMethodeObject = {
    [k in MethodName]: (...arg: any[]) => any;
};
interface IJsonBaseController extends RequestMethodeObject {
}
export declare class JsonBaseController<E extends Entity> implements IJsonBaseController {
    private [TYPEORM_SERVICE_PROPS];
    getOne(id: string | number, query: Query<E>): Promise<ResourceObject<E>>;
    getAll(query: Query<E>): Promise<ResourceObject<E, 'array'>>;
    deleteOne(id: string | number): Promise<void>;
    patchOne(id: string | number, inputData: PatchData<E>): Promise<ResourceObject<E>>;
    postOne(inputData: PostData<E>): Promise<ResourceObject<E>>;
    getRelationship<Rel extends EntityRelation<E>>(id: string | number, relName: Rel): Promise<ResourceObjectRelationships<E, Rel>>;
    postRelationship<Rel extends EntityRelation<E>>(id: string | number, relName: Rel, input: PostRelationshipData): Promise<ResourceObjectRelationships<E, Rel>>;
    deleteRelationship<Rel extends EntityRelation<E>>(id: string | number, relName: Rel, input: PostRelationshipData): Promise<void>;
    patchRelationship<Rel extends EntityRelation<E>>(id: string | number, relName: Rel, input: PatchRelationshipData): Promise<ResourceObjectRelationships<E, Rel>>;
}
export {};
