"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorInterceptors = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const typeorm_1 = require("typeorm");
const DriverUtils_1 = require("typeorm/driver/DriverUtils");
const constants_1 = require("../../constants");
const helper_1 = require("../../helper");
const common_2 = require("@nestjs/common");
let ErrorInterceptors = class ErrorInterceptors {
    repository;
    config;
    intercept(context, next) {
        return next.handle().pipe((0, rxjs_1.catchError)((error) => {
            if (error instanceof typeorm_1.QueryFailedError) {
                return (0, rxjs_1.throwError)(() => this.prepareDataBaseError(error));
            }
            if (error instanceof common_2.HttpException) {
                return (0, rxjs_1.throwError)(() => error);
            }
            const errorObject = {
                code: 'internal_error',
                message: this.config.debug ? error.message : 'Internal Server Error',
                path: [],
            };
            const descriptionOrOptions = this.config.debug ? error : undefined;
            return (0, rxjs_1.throwError)(() => new common_1.InternalServerErrorException([errorObject], descriptionOrOptions));
        }));
    }
    prepareDataBaseError(error) {
        const errorObject = {
            code: 'internal_error',
            message: this.config.debug ? error.message : 'Internal Server Error',
            path: [],
        };
        if (DriverUtils_1.DriverUtils.isMySQLFamily(this.repository.manager.connection.driver)) {
            const { errorCode, errorMsg } = this.prepareMysqlError(error.driverError);
            if (helper_1.MysqlError[errorCode]) {
                return helper_1.MysqlError[errorCode](this.repository.metadata, errorMsg);
            }
        }
        if (DriverUtils_1.DriverUtils.isPostgresFamily(this.repository.manager.connection.driver)) {
            const { errorCode, errorMsg, detail } = this.preparePostgresError(error.driverError);
            if (helper_1.PostgresError[errorCode]) {
                return helper_1.PostgresError[errorCode](this.repository.metadata, errorMsg, detail);
            }
        }
        return new common_1.InternalServerErrorException([errorObject]);
    }
    prepareMysqlError(error) {
        return {
            errorCode: error.errno,
            errorMsg: error.message,
        };
    }
    preparePostgresError(error) {
        return {
            errorCode: error.code,
            errorMsg: error.message,
            detail: error.detail,
        };
    }
};
exports.ErrorInterceptors = ErrorInterceptors;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.CURRENT_ENTITY_REPOSITORY),
    tslib_1.__metadata("design:type", typeorm_1.Repository)
], ErrorInterceptors.prototype, "repository", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.CONTROL_OPTIONS_TOKEN),
    tslib_1.__metadata("design:type", Object)
], ErrorInterceptors.prototype, "config", void 0);
exports.ErrorInterceptors = ErrorInterceptors = tslib_1.__decorate([
    (0, common_1.Injectable)()
], ErrorInterceptors);
//# sourceMappingURL=error.interceptors.js.map