"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModuleClass = void 0;
const helper_1 = require("../../helper");
const constants_1 = require("../../constants");
const factory_1 = require("../../factory");
const service_1 = require("../service");
const service_2 = require("../service");
const swagger_bind_service_1 = require("../service/swagger-bind.service");
class BaseModuleClass {
    static forRoot(options) {
        const { entity, connectionName, controller } = options;
        const controllerClass = (0, helper_1.createController)(entity, controller);
        const decoratorOptions = Reflect.getMetadata(constants_1.JSON_API_DECORATOR_OPTIONS, controllerClass) || {};
        const moduleConfig = {
            ...constants_1.ConfigParamDefault,
            ...options.config,
            ...decoratorOptions,
        };
        (0, helper_1.bindController)(controllerClass, entity, connectionName, moduleConfig);
        const optionProvider = {
            provide: constants_1.CONTROL_OPTIONS_TOKEN,
            useValue: moduleConfig,
        };
        return {
            module: (0, helper_1.nameIt)((0, helper_1.getProviderName)(entity, constants_1.JSON_API_MODULE_POSTFIX), BaseModuleClass),
            controllers: [controllerClass],
            providers: [
                (0, factory_1.ZodInputQuerySchema)(entity),
                (0, factory_1.ZodQuerySchema)(entity),
                (0, factory_1.ZodInputPostSchema)(entity),
                (0, factory_1.ZodInputPatchSchema)(entity),
                factory_1.ZodInputPostRelationshipSchema,
                factory_1.ZodInputPatchRelationshipSchema,
                (0, factory_1.TypeormServiceFactory)(entity),
                (0, factory_1.EntityRepositoryFactory)(entity),
                optionProvider,
                service_1.TypeormUtilsService,
                service_2.TransformDataService,
                swagger_bind_service_1.SwaggerBindService,
            ],
            imports: [],
        };
    }
}
exports.BaseModuleClass = BaseModuleClass;
//# sourceMappingURL=module.mixin.js.map