"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryPipe = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const zod_1 = require("zod");
const helper_1 = require("../../../helper");
const constants_1 = require("../../../constants");
const service_1 = require("../../../service");
class QueryPipe {
    zodQuerySchema;
    transformInputService;
    transform(value) {
        try {
            const { filter, page, sort, include, fields } = value;
            const queryObject = {
                [helper_1.QueryField.filter]: this.transformInputService.transformFilter(filter),
                [helper_1.QueryField.fields]: this.transformInputService.transformFields(fields),
                [helper_1.QueryField.include]: this.transformInputService.transformInclude(include),
                [helper_1.QueryField.sort]: this.transformInputService.transformSort(sort),
                [helper_1.QueryField.page]: page,
            };
            return this.zodQuerySchema.parse(queryObject);
        }
        catch (e) {
            if (e instanceof zod_1.ZodError) {
                throw new common_1.BadRequestException(e.issues);
            }
            throw new common_1.InternalServerErrorException(e);
        }
    }
}
exports.QueryPipe = QueryPipe;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.ZOD_QUERY_SCHEMA),
    tslib_1.__metadata("design:type", Object)
], QueryPipe.prototype, "zodQuerySchema", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(service_1.TransformInputService),
    tslib_1.__metadata("design:type", service_1.TransformInputService)
], QueryPipe.prototype, "transformInputService", void 0);
//# sourceMappingURL=query.pipe.js.map