"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwaggerBindService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const constants_1 = require("@nestjs/common/constants");
const swagger_1 = require("@nestjs/swagger");
const constants_2 = require("@nestjs/swagger/dist/constants");
const json_base_controller_1 = require("../controller/json-base.controller");
const typeorm_1 = require("typeorm");
const constants_3 = require("../../constants");
const helper_1 = require("../../helper");
const bindings_1 = require("../../config/bindings");
let SwaggerBindService = class SwaggerBindService {
    repository;
    discoveryService;
    swaggerMethod;
    config;
    initSwagger() {
        const controllerName = (0, helper_1.nameIt)((0, helper_1.getProviderName)(this.repository.target, constants_3.JSON_API_CONTROLLER_POSTFIX), json_base_controller_1.JsonBaseController).name;
        const controllerInst = this.discoveryService
            .getControllers()
            .find((i) => i.name === controllerName ||
            this.repository.target ===
                Reflect.getMetadata(constants_3.JSON_API_DECORATOR_ENTITY, i.instance.constructor));
        if (!controllerInst)
            throw new Error(`Controller for ${this.repository.metadata.name} is empty`);
        const controller = controllerInst.instance.constructor;
        const apiTag = Reflect.getMetadata(constants_2.DECORATORS.API_TAGS, controller);
        if (!apiTag) {
            (0, swagger_1.ApiTags)(this.config['overrideRoute'] || this.repository.metadata.name)(controller);
        }
        (0, swagger_1.ApiTags)(this.repository.metadata.name)(controller);
        (0, swagger_1.ApiExtraModels)(helper_1.FilterOperand)(controller);
        (0, swagger_1.ApiExtraModels)((0, helper_1.createApiModels)(this.repository))(controller);
        const { allowMethod = Object.keys(bindings_1.Bindings) } = this.config;
        for (const method of helper_1.ObjectTyped.keys(bindings_1.Bindings)) {
            if (!allowMethod.includes(method))
                continue;
            if (method in helper_1.swaggerMethod) {
                helper_1.swaggerMethod[method](controller.prototype, this.repository, bindings_1.Bindings[method], this.config);
            }
            Reflect.defineMetadata(constants_1.PARAMTYPES_METADATA, [Object], controller.prototype, method);
        }
    }
    onModuleInit() {
        this.initSwagger();
    }
};
exports.SwaggerBindService = SwaggerBindService;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_3.CURRENT_ENTITY_REPOSITORY),
    tslib_1.__metadata("design:type", typeorm_1.Repository)
], SwaggerBindService.prototype, "repository", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(core_1.DiscoveryService),
    tslib_1.__metadata("design:type", core_1.DiscoveryService)
], SwaggerBindService.prototype, "discoveryService", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(constants_3.SWAGGER_METHOD),
    tslib_1.__metadata("design:type", Object)
], SwaggerBindService.prototype, "swaggerMethod", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(constants_3.CONTROL_OPTIONS_TOKEN),
    tslib_1.__metadata("design:type", Object)
], SwaggerBindService.prototype, "config", void 0);
exports.SwaggerBindService = SwaggerBindService = tslib_1.__decorate([
    (0, common_1.Injectable)()
], SwaggerBindService);
//# sourceMappingURL=swagger-bind.service.js.map