"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformDataService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const helper_1 = require("../../helper");
const route_path_factory_1 = require("@nestjs/core/router/route-path-factory");
const service_1 = require("../../service");
(0, common_1.Injectable)();
class TransformDataService {
    applicationConfig;
    entityPropsMapService;
    _urlPath;
    get urlPath() {
        if (this._urlPath)
            return [...this._urlPath];
        this._urlPath = [''];
        const prefix = this.applicationConfig.getGlobalPrefix();
        const version = this.applicationConfig.getVersioning();
        const routePathFactory = new route_path_factory_1.RoutePathFactory(this.applicationConfig);
        if (prefix) {
            this._urlPath.push(this.applicationConfig.getGlobalPrefix());
        }
        if (version && version.type === common_1.VersioningType.URI) {
            const firstVersion = Array.isArray(version.defaultVersion)
                ? version.defaultVersion[0]
                : version.defaultVersion;
            if (firstVersion) {
                this._urlPath.push(`${routePathFactory.getVersionPrefix(version)}${firstVersion.toString()}`);
            }
        }
        return [...this._urlPath];
    }
    getLink(...partOfUrl) {
        const urlPath = this.urlPath;
        urlPath.push(...partOfUrl);
        return urlPath.join('/');
    }
    transformData(data) {
        const dataResponse = Array.isArray(data)
            ? data
                .map((i) => this.getMainData(i))
                .filter((i) => !!i)
            : this.getMainData(data);
        let relationships = undefined;
        if (Array.isArray(dataResponse) && dataResponse.length > 0) {
            relationships = dataResponse.reduce((acum, item) => ({
                ...acum,
                ...item.relationships,
            }), {});
        }
        if (!Array.isArray(dataResponse) && dataResponse !== null) {
            relationships = dataResponse.relationships;
        }
        if (relationships === undefined) {
            return { data: dataResponse };
        }
        const propsNeedForInclude = helper_1.ObjectTyped.entries(relationships).reduce((acum, [key, val]) => {
            if (!val || !('data' in val)) {
                return acum;
            }
            if (Array.isArray(val.data)) {
                acum.push(key);
                return acum;
            }
            if (!Array.isArray(val.data)) {
                acum.push(key);
            }
            return acum;
        }, []);
        const included = (Array.isArray(data) ? data : [data])
            .reduce((acum, item) => {
            const tmp = propsNeedForInclude.reduce((a, props) => {
                const currentItem = item[props];
                const r = (Array.isArray(currentItem) ? currentItem : [currentItem]);
                a.push(...r);
                return a;
            }, []);
            acum.push(...tmp);
            return acum;
        }, [])
            .map((i) => {
            return this.getMainData(i);
        })
            .filter((i) => !!i);
        if (included.length > 0) {
            return { data: dataResponse, included };
        }
        return { data: dataResponse };
    }
    getMainData(data) {
        if (!data)
            return null;
        const entity = data.constructor;
        return {
            ...this.getData((0, helper_1.camelToKebab)(this.entityPropsMapService.getNameForEntity(entity)), data[this.entityPropsMapService.getPrimaryColumnsForEntity(entity)]),
            attributes: helper_1.ObjectTyped.entries(data).reduce((acum, [key, val]) => {
                if (this.entityPropsMapService
                    .getRelPropsForEntity(entity)
                    .includes(key.toString()) ||
                    key.toString() ===
                        this.entityPropsMapService.getPrimaryColumnsForEntity(entity)) {
                    return acum;
                }
                acum[key] = val;
                return acum;
            }, {}),
            relationships: this.transformRelationshipsData(data),
            links: {
                self: this.getLink((0, helper_1.camelToKebab)(this.entityPropsMapService.getNameForEntity(entity)), data[this.entityPropsMapService.getPrimaryColumnsForEntity(entity)]),
            },
        };
    }
    getRelationships(data, rel) {
        const entity = data.constructor;
        const relation = data[rel];
        const target = this.entityPropsMapService.getRelationPropsType(entity, rel);
        const typeName = (0, helper_1.camelToKebab)(this.entityPropsMapService.getNameForEntity(target));
        const primaryColumn = this.entityPropsMapService.getPrimaryColumnsForEntity(target);
        const resultData = Array.isArray(relation)
            ? relation.map((i) => this.getData(typeName, i[primaryColumn]))
            : relation === null || !relation
                ? null
                : this.getData(typeName, relation[primaryColumn]);
        return {
            data: resultData,
        };
    }
    transformRelationshipsData(data) {
        const entity = data.constructor;
        return this.entityPropsMapService
            .getRelPropsForEntity(entity)
            .reduce((acum, val) => {
            const result = {
                links: {
                    self: this.getLink((0, helper_1.camelToKebab)(this.entityPropsMapService.getNameForEntity(entity)), data[this.entityPropsMapService.getPrimaryColumnsForEntity(entity)], 'relationships', val),
                },
            };
            if (val in data) {
                const { data: dataRelationships } = this.getRelationships(data, val);
                if (Array.isArray(dataRelationships)) {
                    result['data'] = dataRelationships;
                }
                if (!Array.isArray(dataRelationships)) {
                    result['data'] = dataRelationships;
                }
            }
            acum[val.toString()] = result;
            return acum;
        }, {});
    }
    getData(type, id) {
        return {
            type,
            id: id.toString(),
        };
    }
}
exports.TransformDataService = TransformDataService;
tslib_1.__decorate([
    (0, common_1.Inject)(core_1.ApplicationConfig),
    tslib_1.__metadata("design:type", core_1.ApplicationConfig)
], TransformDataService.prototype, "applicationConfig", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(service_1.EntityPropsMapService),
    tslib_1.__metadata("design:type", service_1.EntityPropsMapService)
], TransformDataService.prototype, "entityPropsMapService", void 0);
//# sourceMappingURL=transform-data.service.js.map