import { Repository } from 'typeorm';
import { RelationMetadata as TypeOrmRelationMetadata } from 'typeorm/metadata/RelationMetadata';
import { Entity, EntityRelation } from '../../types';
import { PatchData, PostData, Query } from '../../helper/zod';
import { TupleOfEntityRelation } from '../../helper/orm';
type RelationAlias<E> = {
    [K in TupleOfEntityRelation<E>[number]]: string;
};
type RelationMetadata<E> = {
    [K in TupleOfEntityRelation<E>[number]]: TypeOrmRelationMetadata;
};
type ResultQueryExpressionObject = {
    name: string;
    val: string;
};
type ResultQueryExpressionArray = {
    name: string;
    val: string;
}[];
export type RelationshipsResult<E extends Entity> = {
    [K in EntityRelation<E>]: E[K] extends E[K][] ? E[K] : E[K] | null;
};
export type ResultQueryExpression = {
    alias?: string;
    expression: string;
    paramsForResult?: string[];
    params?: ResultQueryExpressionObject | ResultQueryExpressionArray;
    selectInclude?: string;
};
export type InputValidateData = {
    type: string;
    id: string;
};
export type ValidateReturn<T> = T extends unknown[] ? string[] : T extends null ? null : string;
export declare class TypeormUtilsService<E extends Entity> {
    private repository;
    private readonly _currentAlias;
    private readonly _relationMetadata;
    private readonly _relationAlias;
    private readonly _relationFields;
    private readonly _entityMetadata;
    private _number;
    constructor(repository: Repository<E>);
    get currentAlias(): string;
    get relationFields(): TupleOfEntityRelation<E>;
    relationName(relName: TupleOfEntityRelation<E>[number]): RelationAlias<E>[TupleOfEntityRelation<E>[number]];
    get currentPrimaryColumn(): keyof E;
    getAliasForRelation(relName: TupleOfEntityRelation<E>[number]): string;
    getRelMetaDataForRelation(relName: TupleOfEntityRelation<E>[number]): RelationMetadata<E>[TupleOfEntityRelation<E>[number]];
    getPrimaryColumnForRel(relName: TupleOfEntityRelation<E>[number]): string;
    private getFilterObject;
    private get number();
    private getParamName;
    getAliasPath(fieldName: unknown): string;
    getAliasPath(fieldName: unknown, relname: TupleOfEntityRelation<E>[number], separator?: string): string;
    getAliasPath(fieldName: unknown, relname: string, separator?: string): string;
    private getSubQueryForManyToMany;
    getFilterExpressionForTarget(query: Query<E>): ResultQueryExpression[];
    getFilterExpressionForRelation(query: Query<E>): ResultQueryExpression[];
    private throwError;
    asyncIterateFindRelationships(relationships: NonNullable<PatchData<E>['relationships'] | PostData<E>['relationships']>): AsyncGenerator<RelationshipsResult<E>>;
    saveEntityData(target: E, relationships: PatchData<E>['relationships'] | PostData<E>['relationships']): Promise<E>;
    validateRelationInputData<Rel extends EntityRelation<E>, In extends InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
    validateRelationInputData<Rel extends EntityRelation<E>, In extends InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
    validateRelationInputData<Rel extends EntityRelation<E>, In extends null | InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
}
export {};
