import { z, ZodArray, ZodLiteral, ZodNullable, ZodNumber, ZodObject, ZodOptional, ZodString, ZodType, ZodTypeDef, ZodUnion } from 'zod';
import { ZodUnionOptions } from 'zod/lib/types';
import { DataSource } from 'typeorm';
import { KEY_MAIN_INPUT_SCHEMA } from '../../constants';
import { MapController } from '../../types';
import { UnionToTupleMain } from '../../../../types';
export declare enum Operation {
    add = "add",
    update = "update",
    remove = "remove"
}
declare const literalSchema: z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodBoolean, z.ZodNull]>;
type Literal = z.infer<typeof literalSchema>;
type Json = Literal | {
    [key: string]: Json;
} | Json[];
type ZodGeneral = ZodNullable<ZodType<Json, ZodTypeDef, Json>>;
export type ZodAdd<T extends string> = ZodObject<{
    op: ZodLiteral<Operation.add>;
    ref: ZodObject<{
        type: ZodLiteral<T>;
        tmpId: ZodOptional<ZodUnion<[ZodNumber, ZodString]>>;
    }>;
    data: ZodGeneral;
}>;
export declare const zodAdd: <T extends string>(type: T) => ZodAdd<T>;
export type ZodUpdate<T extends string> = ZodObject<{
    op: ZodLiteral<Operation.update>;
    ref: ZodObject<{
        type: ZodLiteral<T>;
        id: ZodString;
    }>;
    data: ZodGeneral;
}>;
export declare const zodUpdate: <T extends string>(type: T) => ZodUpdate<T>;
export type ZodRemove<T extends string> = ZodObject<{
    op: ZodLiteral<Operation.remove>;
    ref: ZodObject<{
        type: ZodLiteral<T>;
        id: ZodString;
    }>;
}>;
export declare const zodRemove: <T extends string>(type: T) => ZodRemove<T>;
type RelToLiteralArray<Rel extends [string, ...string[]]> = UnionToTupleMain<{
    [K in Rel[number]]: ZodLiteral<K>;
}[Rel[number]]>;
type RelLiteralArrayToUnion<T> = T extends ZodUnionOptions ? ZodUnion<T> : never;
type ZodRelLiteral<Rel extends [string, ...string[]]> = RelLiteralArrayToUnion<RelToLiteralArray<Rel>>;
export type ZodOperationRel<T extends string, Rel extends [string, ...string[]], OP extends Operation> = ZodObject<{
    op: ZodLiteral<OP>;
    ref: ZodObject<{
        type: ZodLiteral<T>;
        id: ZodString;
        relationship: ZodRelLiteral<Rel>;
    }>;
    data: ZodGeneral;
}>;
export declare const zodOperationRel: <T extends string, R extends [string, ...string[]], OP extends Operation>(type: T, rel: R, typeOperation: OP) => ZodOperationRel<T, R, OP>;
export type ZodInputArray = ZodArray<ZodObject<{
    op: ZodLiteral<Operation>;
    ref: ZodObject<{
        type: ZodString;
        id: ZodOptional<ZodString>;
        relationship: ZodOptional<ZodString>;
        tmpId: ZodOptional<ZodUnion<[ZodNumber, ZodString]>>;
    }>;
    data: ZodOptional<ZodGeneral>;
}>, 'atleastone'>;
export type InputArray = z.infer<ZodInputArray>;
export type ZodInputOperation = ZodObject<{
    [KEY_MAIN_INPUT_SCHEMA]: ZodInputArray;
}, 'strict'>;
export declare const zodInputOperation: (dataSource: DataSource, mapController: MapController) => ZodInputOperation;
export {};
