"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityPropsMapService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const typeorm_1 = require("typeorm");
const constants_1 = require("../constants");
const orm_1 = require("../helper/orm");
let EntityPropsMapService = class EntityPropsMapService {
    dataSource;
    _propsForEntity = new Map();
    _relPropsForEntity = new Map();
    _relTypePropsForEntity = new Map();
    _primaryColumnsForEntity = new Map();
    _nameForEntity = new Map();
    getPropsForEntity(entity) {
        const result = this._propsForEntity.get(entity);
        if (result) {
            return result;
        }
        const { field } = this.pullPropsAndRelFoEntity(entity);
        return field;
    }
    getRelPropsForEntity(entity) {
        const result = this._relPropsForEntity.get(entity);
        if (result) {
            return result;
        }
        const { relations } = this.pullPropsAndRelFoEntity(entity);
        return relations;
    }
    getRelationPropsType(entity, rel) {
        const result = this._relTypePropsForEntity.get(entity);
        if (result) {
            return result[rel];
        }
        const repo = this.dataSource.getRepository(entity);
        const relToType = repo.metadata.relations.reduce((acum, item) => {
            acum[item.propertyName] = item.inverseEntityMetadata.target;
            return acum;
        }, {});
        this._relTypePropsForEntity.set(entity, relToType);
        return relToType[rel];
    }
    getPrimaryColumnsForEntity(entity) {
        const result = this._primaryColumnsForEntity.get(entity);
        if (result) {
            return result;
        }
        const primaryColumns = this.dataSource.getRepository(entity).metadata.primaryColumns[0].propertyName;
        this._primaryColumnsForEntity.set(entity, primaryColumns);
        return primaryColumns;
    }
    getNameForEntity(entity) {
        const result = this._nameForEntity.get(entity);
        if (result) {
            return result;
        }
        const name = this.dataSource.getRepository(entity)
            .metadata.name;
        this._nameForEntity.set(entity, name);
        return name;
    }
    pullPropsAndRelFoEntity(entity) {
        const repo = this.dataSource.getRepository(entity);
        const { relations, field } = (0, orm_1.getField)(repo);
        this._propsForEntity.set(entity, field);
        this._relPropsForEntity.set(entity, relations);
        return { relations, field };
    }
};
exports.EntityPropsMapService = EntityPropsMapService;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.CURRENT_DATA_SOURCE_TOKEN),
    tslib_1.__metadata("design:type", typeorm_1.DataSource)
], EntityPropsMapService.prototype, "dataSource", void 0);
exports.EntityPropsMapService = EntityPropsMapService = tslib_1.__decorate([
    (0, common_1.Injectable)()
], EntityPropsMapService);
//# sourceMappingURL=entity-props-map.service.js.map