"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformInputService = void 0;
const common_1 = require("@nestjs/common");
const types_1 = require("../types");
const utils_1 = require("../helper/utils");
const constants_1 = require("../constants");
const arrayOp = {
    [types_1.FilterOperand.in]: true,
    [types_1.FilterOperand.nin]: true,
    [types_1.FilterOperand.some]: true,
};
function convertToFilterObject(value) {
    if ((0, utils_1.isString)(value)) {
        return {
            [types_1.FilterOperand.eq]: value,
        };
    }
    else {
        return Object.entries(value).reduce((acum, [op, filed]) => {
            if (op in arrayOp) {
                acum[op] = ((0, utils_1.isString)(filed) ? filed.split(',') : []).filter((i) => !!i);
            }
            else {
                acum[op] = filed;
            }
            return acum;
        }, {});
    }
}
(0, common_1.Injectable)();
class TransformInputService {
    transformSort(data) {
        if (!data)
            return null;
        return data
            .split(',')
            .map((i) => i.trim())
            .filter((i) => !!i)
            .reduce((acum, field) => {
            const fieldName = field.charAt(0) === '-' ? field.substring(1) : field;
            const sort = field.charAt(0) === '-' ? constants_1.DESC : constants_1.ASC;
            if (fieldName.indexOf('.') > -1) {
                const [relation, fieldRelation] = field.split('.');
                const relationName = relation.charAt(0) === '-' ? relation.substring(1) : relation;
                acum[relationName] = acum[relationName] || {};
                acum[relationName][fieldRelation] = sort;
            }
            else {
                acum['target'] = acum['target'] || {};
                acum['target'][fieldName] = sort;
            }
            return acum;
        }, {});
    }
    transformInclude(data) {
        if (!data || !(0, utils_1.isString)(data))
            return null;
        return data
            .split(',')
            .map((i) => i.trim())
            .filter((i) => !!i);
    }
    transformFields(data) {
        if (!data)
            return null;
        const prepareResult = Object.entries(data).reduce((acum, [key, val]) => {
            acum[key] = val
                .split(',')
                .map((i) => i.trim())
                .filter((i) => !!i);
            return acum;
        }, {});
        const result = Object.entries(prepareResult).reduce((acum, [key, value]) => {
            if (value.length > 0) {
                acum[key] = value;
            }
            return acum;
        }, {});
        return Object.keys(result).length > 0 ? result : null;
    }
    transformFilter(data) {
        if (!data) {
            return {
                relation: null,
                target: null,
            };
        }
        return Object.entries(data).reduce((acum, [field, value]) => {
            const objectOperand = convertToFilterObject(value);
            if (Object.keys(objectOperand).length === 0) {
                return acum;
            }
            if (field.indexOf('.') > -1) {
                const [relation, fieldRelation] = field.split('.');
                acum['relation'] = !acum['relation'] ? {} : acum['relation'];
                acum['relation'][relation] = acum['relation'][relation] || {};
                acum['relation'][relation][fieldRelation] = objectOperand;
            }
            else {
                acum['target'] = !acum['target'] ? {} : acum['target'];
                acum['target'][field] = objectOperand;
            }
            return acum;
        }, { relation: null, target: null });
    }
}
exports.TransformInputService = TransformInputService;
//# sourceMappingURL=transform-input.service.js.map